/*
Copyright (C) 2024 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR

This file is part of simcaq-node.

simcaq-node is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

simcaq-node is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with simcaq-node.  If not, see <https://www.gnu.org/licenses/>.
*/

const express = require('express');

const ExpensesApp = express.Router();

const libs = `${process.cwd()}/libs`;

const squel = require('squel');

const query = require(`${libs}/middlewares/query`).query;

const response = require(`${libs}/middlewares/response`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

const id2str = require(`${libs}/middlewares/id2str`);

const config = require(`${libs}/config`); 

const cache = require('apicache').options({ debug: config.debug, statusCodes: {include: [200]} }).middleware;

const aggregateData = require(`${libs}/middlewares/aggregateData`);

let rqf = new ReqQueryFields();

ExpensesApp.use(cache('15 day'));

ExpensesApp.get('/years', (req, res, next) => {
    req.sql.from('despesas')
    .field('DISTINCT despesas.ano_censo', 'year')
    .where('despesas.ano_censo is not null')
    next();
}, query, response('years'))

ExpensesApp.get('/type', (req, res, next) => {
    req.result = []

    req.result.push({ id: 1, name: "Rede Estadual" });
    req.result.push({ id: 2, name: "Rede Municipal" });
    next();
}, response('type'));

// rqf.addValue({
//     name: 'location',
//     table: 'despesas',
//     tableField: 'nome_ente',
//     resultField: 'location_id',
//     where: {
//         relation: '=',
//         type: 'integer',
//         field: 'localizacao_id'
//     }
// });

rqf.addField({
    name: 'filter',
    field: false,
    where: true
}).addField({
    name: 'dims',
    field: true,
    where: false
}).addValue({
    name: 'min_year',
    table: 'despesas',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '>=',
        type: 'integer',
        field: 'ano_censo'
    }
}).addValue({
    name: 'max_year',
    table: 'despesas',
    tableField: 'ano_censo',
    resultField: 'year',
    where: {
        relation: '<=',
        type: 'integer',
        field: 'ano_censo'
    }
}).addValue({
    name: 'region',
    table: 'estado',
    tableField: ['nome_ente', 'cod_ibge', 'id', 'regiao_id'],
    resultField: ['state_name', 'state_cod_ibge', 'state_id', 'region_id'],
    where: {
        relation: '=',
        type: 'integer',
        field: 'regiao_id',
    },
    join: {
        primary: 'id',
        foreign: 'cod_ibge',
        foreignTable: 'despesas'
    }
})


ExpensesApp.get(
  '/',
  rqf.parse(),
  rqf.build(),
  (req, res, next) => {
    let obj = {};
    let typeFilter = false;
    // condição inicial padrão (sem filtro de região, coloca cod_ibge = 0 só para não quebrar a query)
    let whereCondition = req.query.filter.includes('region')
      ? ''
      : 'despesas.cod_ibge = 0';
    let filterId;
    const filters = req.query.filter.split(',');

    // Se houver filtro de estado/cidade, ajusta o cod_ibge
    filters.forEach((filter) => {
      if (filter.includes('state') || filter.includes('city')) {
        filterId = Number(filter.split(':')[1].replace(/"/g, ''));
        whereCondition = `despesas.cod_ibge = ${filterId}`;
      }
    });

    // Interpreta o JSON de filtros; se tiver “type”, ajusta o where para agregar por tipo
    if (req.query.filter) {
      const jsonString = `{${req.query.filter.replace(/(\w+):/g, '"$1":')}}`;
      obj = JSON.parse(jsonString);

      if (obj.type) {
        // Se vier só “1” (por exemplo, Unidades Federativas)
        if (obj.type.includes('1') && !obj.type.includes('2')) {
          whereCondition = 'despesas.cod_ibge >= 11 AND despesas.cod_ibge <= 53';
        }
        // Se vier só “2” (Municípios)
        else if (obj.type.includes('2') && !obj.type.includes('1')) {
          whereCondition = 'despesas.cod_ibge > 53';
        }
        typeFilter = true;
      }
    }

    req.sql
      .from('despesas')
      .field('despesas.ano_censo', 'year')
      .field('SUM(ROUND(despesas.despesas, 2))', 'total')
      .where(whereCondition)
      .group('despesas.ano_censo')
      .order('despesas.ano_censo');

    next();
  },
  query,
  id2str.transform(false),
  response('expenses')
);
module.exports = ExpensesApp;